﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using Microsoft.ServiceHosting.ServiceRuntime;
// Added for Azure Table Services
using Microsoft.Samples.ServiceHosting.StorageClient;
using System.Collections.Generic;
using System.Data.Services.Client;
using System.Diagnostics;

// See "Deploying a Service on Windows Azure" by Jim Nakashima (http://msdn.microsoft.com/en-us/library/dd203057.aspx)
// to deploy this project to Windows Azure

namespace SampleWebCloudService_WebRole
{
    public partial class _Default : System.Web.UI.Page
    {
        // Used by multiple handlers
        StorageAccountInfo account = null;
        CustomerDataServiceContext serviceContext = null;
        public bool skipPreRenderTime = false;

        protected void Page_Load(object sender, EventArgs e)
        {
            // Get the StorageAccountInfo once            
            account = StorageAccountInfo.GetDefaultTableStorageAccountFromConfiguration();
            serviceContext = new CustomerDataServiceContext(account);
            serviceContext.RetryPolicy = RetryPolicies.RetryN(3, TimeSpan.FromSeconds(1));
        }

        protected void Page_Prerender(object sender, EventArgs e)
        {
            // This query gets a page of 12 customers at a time
            // See Paging Over Data in Windows Azure Tables by Steve Marx 
            // http://blog.smarx.com/posts/paging-over-data-in-windows-azure-tables
            Stopwatch timer = new Stopwatch();
            timer.Start();

            // Write a LINQ to REST query
            int pageSize = 12;
            var query = (DataServiceQuery<CustomerDataModel>)
                (new CustomerDataServiceContext(account).CustomerTable.Take(pageSize));
            
            var continuation = Request["ct"];
            if (continuation != null)
            {
                // ct param looks like "<partition>/<row>"
                string[] tokens = continuation.Split('/');
                var partitionToken = tokens[0];
                var rowToken = tokens[1];

                // These become continuation token (ct) query parameters in the request.
                query = query.AddQueryOption("NextPartitionKey", partitionToken)
                             .AddQueryOption("NextRowKey", rowToken);
            }
            var result = query.Execute();

            // Cast to a QueryOperationResponse and read the custom headers
            var qor = (QueryOperationResponse)result;
            string nextPartition = null;
            string nextRow = null;
            qor.Headers.TryGetValue("x-ms-continuation-NextPartitionKey", 
                out nextPartition);
            qor.Headers.TryGetValue("x-ms-continuation-NextRowKey", 
                out nextRow);

            if (nextPartition != null && nextRow != null)
            {
                nextLink.NavigateUrl = string.Format("?ct={0}/{1}", nextPartition, 
                    nextRow);
                txtNextPartitionKey.Text = nextPartition;
                txtNextRowKey.Text = nextRow;
                btnCreateCustomers.Enabled = false;
                btnDeleteAll.Enabled = true;
            }
            else
            {
                txtNextPartitionKey.Text = null;
                txtNextRowKey.Text = null;
                nextLink.Visible = false;
            }
            // Change the customersView DataView's DataSource to the query response
            customersView.DataSourceID = null;
            customersView.DataSource = result;
            customersView.DataBind();
            if (!skipPreRenderTime)
            {
                txtTime.Text = (timer.ElapsedMilliseconds / 1000D).ToString("0.000");
                btnUpdateCusts.Enabled = true;
            }
            skipPreRenderTime = false;
        }
    
        // Iterates the CustomerTable to return a count of entities
        protected void btnQuery_Click(object sender, EventArgs e)
        {
            Stopwatch timer = new Stopwatch();
            timer.Start();

            var results = from c in serviceContext.CustomerTable
                          select c;
            TableStorageDataServiceQuery<CustomerDataModel> query = 
                new TableStorageDataServiceQuery<CustomerDataModel>
                    (results as DataServiceQuery<CustomerDataModel>);
            IEnumerable<CustomerDataModel> queryResults = query.ExecuteAllWithRetries();

            // The Count() function isn't implemented
            int count = 0;
            foreach (var result in results)
            {
                count +=1;
            }
            txtCount.Text = count.ToString();
            txtTime.Text = (timer.ElapsedMilliseconds / 1000D).ToString("0.000");
        }

        // Deletes all CustomerTable entities individually
        // Sets the customersView GridView's DataSource 
        protected void btnDeleteAll_Click(object sender, EventArgs e)
        {
            Stopwatch timer = new Stopwatch();
            timer.Start();
            skipPreRenderTime = true;            
            var results = from c in serviceContext.CustomerTable
                          select c;
            TableStorageDataServiceQuery<CustomerDataModel> query = 
                new TableStorageDataServiceQuery<CustomerDataModel>
                    (results as DataServiceQuery<CustomerDataModel>);
            IEnumerable<CustomerDataModel> queryResults = query.ExecuteAllWithRetries();

            foreach (var result in results)
            {
                serviceContext.DeleteObject(result);
            }
            serviceContext.SaveChanges();
            customersView.DataSource = null;
            customersView.DataSourceID = "customerData";
            customersView.DataBind();

            btnCreateCustomers.Enabled = true;
            btnUpdateCusts.Enabled = false;
            txtTime.Text = (timer.ElapsedMilliseconds / 1000D).ToString("0.000");
        }

        // Creates all CustomerTable entities individually from the object initializers
        // Sets the customersView GridView's DataSource 
        protected void btnCreateCustomers_Click(object sender, EventArgs e)
        {
            Stopwatch timer = new Stopwatch();
            timer.Start();

            skipPreRenderTime = true;            
            List<CustomerDataModel> custDataModel = null;
            if (chkRowKeyPK.Checked)
                custDataModel = CreateCustomerListWithRowKey();
            else
                custDataModel = CreateCustomerList();

            foreach (CustomerDataModel newItem in custDataModel)
            {
                if (newItem.CustomerID == null)
                    newItem.CustomerID = newItem.RowKey;

                serviceContext.AddObject(CustomerDataServiceContext.CustomerTableName, 
                    newItem);
            }
            serviceContext.SaveChanges();
            customersView.DataSource = null;
            customersView.DataSourceID = "customerData";
            customersView.DataBind();

            txtTime.Text = (timer.ElapsedMilliseconds / 1000D).ToString("0.000");
            btnCreateCustomers.Enabled = false;
            btnUpdateCusts.Enabled = true;
        }

        // Updates all CustomerTable entities individually
        // Sets the customersView GridView's DataSource 
        protected void btnUpdateCusts_Click(object sender, EventArgs e)
        {
            Stopwatch timer = new Stopwatch();
            timer.Start();
            skipPreRenderTime = true;            

            CustomerDataServiceContext serviceContext = null;

            // Create the service context to query against
            serviceContext = new CustomerDataServiceContext(account);
            serviceContext.RetryPolicy = RetryPolicies.RetryN(3, 
                TimeSpan.FromSeconds(1));

            var results = from c in serviceContext.CustomerTable
                          select c;
            TableStorageDataServiceQuery<CustomerDataModel> query = 
                new TableStorageDataServiceQuery<CustomerDataModel>
                    (results as DataServiceQuery<CustomerDataModel>);
            IEnumerable<CustomerDataModel> queryResults = 
                query.ExecuteAllWithRetries();

            // Toggle them as updated 
            foreach (var result in results)
            {
                if (result.CompanyName.Contains("+"))
                    result.CompanyName = result.CompanyName.Substring(0, result.CompanyName.Length - 1);
                else
                    result.CompanyName += "+";
                serviceContext.UpdateObject(result);
            }
            serviceContext.SaveChanges();
            
            customersView.DataSource = null;
            customersView.DataSourceID = "customerData";
            customersView.DataBind();
            txtTime.Text = (timer.ElapsedMilliseconds / 1000D).ToString("0.000");
        }

        protected void frmAdd_PageIndexChanging(object sender, FormViewPageEventArgs e)
        {

        }
    }
}
